/*
 * Copyright (c) 2011 Stefan Bolte <portix@gmx.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <locale.h>
#include <sys/stat.h>
#include <termios.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#define BUFFER_SIZE  256 

int main(int argc, char **argv) {
  XEvent e;
  Display *dpy; 
  Window focus;
  int ret, flags, n;
  char buffer[BUFFER_SIZE + 1];
  KeySym ks;
  char *keysym, *last;
  setlocale(LC_ALL, "");

  if (argc > 1) {
    if (!strncmp(argv[1], "-h", 2)) {
      printf("Usage: %s [-h | -v]\n", NAME);
    }
    if (!strncmp(argv[1], "-v", 2)) {
      printf("%s version: %s\n", NAME, VERSION);
    }
    return 0;
  }
  if ( ! (dpy = XOpenDisplay(NULL))) {
    fprintf(stderr, "Cannot open display.\n");
    return 1;
  }
  XGetInputFocus(dpy, &focus, &ret);
  XSelectInput(dpy, focus, KeyPressMask );

  struct termios attr;
  tcgetattr(0, &attr);
  flags = attr.c_lflag;
  attr.c_lflag &= ~(ECHO);
  tcsetattr(0, TCSANOW, &attr);


  printf("press any key ('Escape' twice exits)...\n\n");
  while (1) {
    XNextEvent(dpy, &e);
    if (e.type == KeyPress) {
      n = XLookupString(&e.xkey, buffer, 256, &ks, NULL);
      buffer[n<0 ? 0 : n > BUFFER_SIZE ? BUFFER_SIZE : n] = '\0';
      keysym = XKeysymToString(ks);

      if (!strcmp(last, "Escape")) {
        attr.c_lflag = flags;
        tcsetattr(0, TCSAFLUSH, &attr);
        XCloseDisplay(dpy);
        return 0;
      }
      else {
        buffer[n > 256 ? 256 : n < 0 ? 0 : n] = '\0';
        if (buffer[0] == '\r') 
          buffer[0] = '\0';
        printf("Symbol: %s \tkeycode: %d \tkeysym: %s\n", buffer, e.xkey.keycode, keysym);
      }
      last = keysym;
    }
  }
}
